"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.defaultSettings = void 0;
const node_1 = require("vscode-languageserver/node");
const fs = __importStar(require("fs"));
const fast_glob_1 = __importDefault(require("fast-glob"));
const culori = __importStar(require("culori"));
const axios_1 = __importDefault(require("axios"));
const postcss_1 = __importDefault(require("postcss"));
const url_1 = require("url");
const path_1 = __importDefault(require("path"));
const postcss_scss_1 = __importDefault(require("postcss-scss"));
const postcss_less_1 = __importDefault(require("postcss-less"));
const CacheManager_1 = __importDefault(require("./CacheManager"));
const isColor_1 = __importDefault(require("./utils/isColor"));
const culoriColorToVscodeColor_1 = require("./utils/culoriColorToVscodeColor");
// The global settings, used when the `workspace/configuration` request is not supported by the client.
// Please note that this is not the case when using this server with the client provided in this example
// but could happen with other clients.
exports.defaultSettings = {
    lookupFiles: ['**/*.less', '**/*.scss', '**/*.sass', '**/*.css'],
    blacklistFolders: [
        '**/.cache',
        '**/.DS_Store',
        '**/.git',
        '**/.hg',
        '**/.next',
        '**/.svn',
        '**/bower_components',
        '**/CVS',
        '**/dist',
        '**/node_modules',
        '**/tests',
        '**/tmp',
    ],
};
const getAST = (filePath, content) => {
    const fileExtension = path_1.default.extname(filePath);
    if (fileExtension === '.less') {
        return postcss_less_1.default.parse(content);
    }
    if (fileExtension === '.scss') {
        return postcss_scss_1.default.parse(content);
    }
    return postcss_1.default.parse(content);
};
class CSSVariableManager {
    constructor() {
        this.cacheManager = new CacheManager_1.default();
        this.parseCSSVariablesFromText = async ({ content, filePath, }) => {
            try {
                // reset cache for this file
                this.cacheManager.clearFileCache(filePath);
                const ast = getAST(filePath, content);
                const fileURI = (0, url_1.pathToFileURL)(filePath).toString();
                const importUrls = [];
                ast.walkAtRules((atRule) => {
                    if (atRule.name === 'import') {
                        // only support absolute url for now
                        const match = atRule.params.match(/['"](?<protocol>http|https):\/\/(?<url>.*?)['"]/);
                        if (match) {
                            const url = `${match.groups.protocol}://${match.groups.url}`;
                            importUrls.push(url);
                        }
                    }
                });
                await Promise.all(importUrls.map(async (url) => {
                    try {
                        const response = await (0, axios_1.default)(url, {
                            responseType: 'text',
                        });
                        const cssText = await response.data;
                        return this.parseCSSVariablesFromText({
                            content: cssText,
                            filePath: url,
                        });
                    }
                    catch (err) {
                        console.log(err, `cannot fetch data from ${url}`);
                    }
                }));
                ast.walkDecls((decl) => {
                    if (decl.prop.startsWith('--')) {
                        const variable = {
                            symbol: {
                                name: decl.prop,
                                value: decl.value,
                            },
                            definition: {
                                uri: fileURI,
                                range: node_1.Range.create(node_1.Position.create(decl.source.start.line - 1, decl.source.start.column - 1), node_1.Position.create(decl.source.end.line - 1, decl.source.end.column - 1)),
                            },
                        };
                        if ((0, isColor_1.default)(decl.value)) {
                            const culoriColor = culori.parse(decl.value);
                            if (culoriColor) {
                                variable.color = (0, culoriColorToVscodeColor_1.culoriColorToVscodeColor)(culoriColor);
                            }
                        }
                        // add to cache
                        this.cacheManager.set(filePath, decl.prop, variable);
                    }
                });
            }
            catch (error) {
                console.error(filePath);
            }
        };
        this.parseAndSyncVariables = async (workspaceFolders, settings = exports.defaultSettings) => {
            for (const folderPath of workspaceFolders) {
                await (0, fast_glob_1.default)(settings.lookupFiles, {
                    onlyFiles: true,
                    cwd: folderPath,
                    ignore: settings.blacklistFolders,
                    absolute: true,
                }).then((files) => {
                    return Promise.all(files.map((filePath) => {
                        const content = fs.readFileSync(filePath, 'utf8');
                        return this.parseCSSVariablesFromText({
                            content,
                            filePath,
                        });
                    }));
                });
            }
        };
    }
    getAll() {
        return this.cacheManager.getAll();
    }
    clearFileCache(filePath) {
        this.cacheManager.clearFileCache(filePath);
    }
    clearAllCache() {
        this.cacheManager.clearAllCache();
    }
}
exports.default = CSSVariableManager;
//# sourceMappingURL=CSSVariableManager.js.map