"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizeFsPath = exports.normalizePath = exports.uriToPath = void 0;
const vscode_uri_1 = require("vscode-uri");
const RE_PATHSEP_WINDOWS = /\\/g;
function uriToPath(stringUri) {
    const uri = vscode_uri_1.URI.parse(stringUri);
    if (uri.scheme !== 'file') {
        return undefined;
    }
    return normalizeFsPath(uri.fsPath);
}
exports.uriToPath = uriToPath;
/**
 * Normalizes the file system path.
 *
 * On systems other than Windows it should be an no-op.
 *
 * On Windows, an input path in a format like "C:/path/file.ts"
 * will be normalized to "c:/path/file.ts".
 */
function normalizePath(filePath) {
    const fsPath = vscode_uri_1.URI.file(filePath).fsPath;
    return normalizeFsPath(fsPath);
}
exports.normalizePath = normalizePath;
/**
 * Normalizes the path obtained through the "fsPath" property of the URI module.
 */
function normalizeFsPath(fsPath) {
    return fsPath.replace(RE_PATHSEP_WINDOWS, '/');
}
exports.normalizeFsPath = normalizeFsPath;
//# sourceMappingURL=protocol.js.map